/**
 * Compact events are to be delivered when:
 * 1. the application is in idle state for long duration
 * 2. the application had been deactivated ( application has lost the focus )
 * 3. the application had been minimized
 * these are to reduce memory usage, like garbage collection, cache cleaning,
 * or etc ...
 */
package jp.kirikiri.tvp2.base;

import java.util.ArrayList;

import jp.kirikiri.tjs2.TJSException;
import jp.kirikiri.tjs2.TJSScriptError;
import jp.kirikiri.tjs2.TJSScriptException;

/**
 *
 */
public class CompactEvent {
	private ArrayList<CompactEventCallbackInterface> mCompactEventVector;
	public CompactEvent() {
		mCompactEventVector = new ArrayList<CompactEventCallbackInterface>();
	}
	public void addCompactEventHook(CompactEventCallbackInterface callback) {
		mCompactEventVector.add(callback);
	}
	public void removeCompactEventHook(CompactEventCallbackInterface callback) {
		final int count = mCompactEventVector.size();
		for( int i = 0; i < count; i++ ) {
			CompactEventCallbackInterface itf = mCompactEventVector.get(i);
			if( itf == callback ) {
				mCompactEventVector.set( i, null );
			}
		}
	}
	public void deliverCompactEvent( int level ) {
		// must be called by each platforms's implementation
		final int count = mCompactEventVector.size();
		if( count > 0 ) {
			boolean emptyflag = false;
			for( int i = 0; i < count; i ++) {
				// note that the handler can remove itself while the event
				try {
					try {
						CompactEventCallbackInterface itf = mCompactEventVector.get(i);
						if( itf != null )
							itf.onCompact(level);
						else
							emptyflag = true;
					} catch( TJSScriptException e ) {
						throw e;
					} catch( TJSScriptError e ) {
						throw e;
					} catch( TJSException e) {
						throw e;
					} catch( Exception e ) {
						throw new TJSException(e.getMessage());
					}
				} catch(TJSScriptError e) {
					e.addTrace( "Compact Event" );
					ScriptsClass.showScriptException(e);
				} catch( TJSException e ) {
					ScriptsClass.showScriptException(e);
				}
			}

			if( emptyflag ) {
				// the array has empty cell
				// eliminate empty
				for( int i = count-1; i >= 0; i-- ) {
					CompactEventCallbackInterface itf = mCompactEventVector.get(i);
					if( itf == null ) mCompactEventVector.remove(i);
				}
			}
		}
	}
}
